/*
 * ToGrammar.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_TO_GRAMMAR_H__
#define _AUTOMATON_TO_GRAMMAR_H__

#include <grammar/Regular/LeftRG.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <grammar/Grammar.h>
#include <automaton/Automaton.h>

namespace automaton {

namespace convert {

class ToGrammar : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	ToGrammar() {}

	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static grammar::Grammar convert(const automaton::Automaton& automaton);

private:
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;
	void Visit(void*, const DFTA& automaton) const;
	void Visit(void*, const NFTA& automaton) const;

	static const ToGrammar TO_GRAMMAR;
};

} /* namespace convert */

} /* namespace automaton */

#endif /* _AUTOMATON_TO_GRAMMAR_H__ */
