/*
 * RandomStringFactory.cpp
 *
 *  Created on: 27. 3. 2014
 *	  Author: Jan Travnicek
 */

#include "RandomStringFactory.h"

#include <algorithm>

namespace string {

namespace generate {

string::LinearString RandomStringFactory::generateLinearString( size_t size, size_t alphabetSize ) {
	srand( time( NULL ) );

	if(alphabetSize > 26)
		throw exception::AlibException("Too big alphabet.");

	if( alphabetSize <= 0 )
		throw exception::AlibException( "Alphabet size must be greater than 0." );

	std::set<alphabet::Symbol> alphabet;
	while( alphabet.size( ) < alphabetSize ) {
		std::string s( 1, rand() % 26 + 'a' );
		alphabet::Symbol symbol = alphabet::symbolFrom (s);
		alphabet.insert( symbol );
	}

	return RandomStringFactory::generateLinearString( size, alphabet );
}

string::LinearString RandomStringFactory::generateLinearString( size_t size, std::set<alphabet::Symbol> alphabet) {

	if( alphabet.size() > 26)
		throw exception::AlibException("Too big alphabet.");

	if( alphabet.size() <= 0 )
		throw exception::AlibException( "Alphabet size must be greater than 0." );

	std::vector<alphabet::Symbol> alphabetList(alphabet.begin(), alphabet.end());
	std::vector<alphabet::Symbol> elems;

	for(size_t i = 0; i < size; i++) {
		elems.push_back(alphabetList[rand() % alphabetList.size()]);
	}

	return string::LinearString(elems);
}

} /* namespace generate */

} /* namespace string */
