/*
 * ExactFactorAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactFactorAutomaton.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactFactorAutomaton::construct(const string::String& text) {
	automaton::Automaton* out = NULL;
	text.getData().Accept((void*) &out, ExactFactorAutomaton::EXACT_FACTOR_AUTOMATON);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

automaton::EpsilonNFA ExactFactorAutomaton::construct(const string::LinearString& text) {
	automaton::EpsilonNFA res(automaton::State(0));
	res.addFinalState(automaton::State(0));
	res.setInputAlphabet(text.getAlphabet());
	int i = 1;
	for(const alphabet::Symbol& symbol : text.getContent()) {
		res.addState(automaton::State(i));
		res.addFinalState(automaton::State(i));
		res.addTransition(automaton::State(i-1), symbol, automaton::State(i));
		res.addTransition(automaton::State(0), automaton::State(i));
		i++;
	}
	return res;
}

void ExactFactorAutomaton::Visit(void* data, const string::Epsilon& text) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->construct(string::LinearString(text)));
}

void ExactFactorAutomaton::Visit(void* data, const string::LinearString& text) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->construct(text));
}

void ExactFactorAutomaton::Visit(void*, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported string type CyclicString");
}

const ExactFactorAutomaton ExactFactorAutomaton::EXACT_FACTOR_AUTOMATON;

} /* namespace exact */

} /* namespace stringology */
